# aceme.py: first release
# EME antenna controller by Harke Smits, PA0HRK
# Version 2.1: works with sensor system based on Arduino by Hans ON4CDU
# 2016-07-25: Quit button implemented, cursur suppressed
# 2016-05-25: runs automatically from LXDE: remove v21 from filename!
# 2016-07-06
# Station specific data has to be adapted, if necessary in: acemedat.py only! Must be in same directory!
# NB: run as sudo!

from Tkinter import *		# graphic module
from math import pi         # math module
import Adafruit_BBIO.UART as UART    # requires ROOT permission!
import Adafruit_BBIO.GPIO as GPIO    # idem, dig I/O 
import struct               # data structuring module 
import time	                # time modules
import datetime
import serial               # serial control module
import ephem                # ephemeris module
import DMCC                 # motor control 
import acemedat             # contains station specific data
import sys 

fnt=("Arial 12 bold")		# standard text font/size

UART.setup("UART2")         # init UART2 for GPS module, not used
ser2 = serial.Serial(port="/dev/ttyO2")
ser2.close

UART.setup("UART4")         # init UART4 to read sensor data (this case we only read)
delim = ['\xd2']            # dataframe start byte Arduino system by ON4CDU
termin = ['\x13']           # dataframe terminator
ser4 = serial.Serial(port="/dev/ttyO4", timeout=1)
ser4.close()                # define port state

# button definitions: default ground = zero state 
dwn = "P8_11"
up  = "P8_12"
lft = "P8_16"
rgt = "P8_17"
stp = "P8_18"
GPIO.setup(dwn,GPIO.IN)
GPIO.setup(up,GPIO.IN)
GPIO.setup(stp,GPIO.IN)  # Auto Manual button: renamed Stop 
GPIO.add_event_detect(stp,GPIO.RISING)
GPIO.setup(lft,GPIO.IN)
GPIO.setup(rgt,GPIO.IN)

ob=ephem.Observer()
# 
# Always get the latest ISS TLE data from:
# http://spaceflight.nasa.gov/realdata/sightings/SSapplications/Post/JavaSSOP/orbit/ISS/SVPOST.html
iss = ephem.readtle('ISS',
    ' 1 25544U 98067A   16148.50606296  .00016717  00000-0  10270-3 0  9001',
    ' 2 25544  51.6409 161.2087 0001609 124.5472 235.5834 15.54704009  1762')

manual = True
space_buttons = []          # empty space button list 

root = Tk()                 # gui module 
root.title("EME Antenna Controller (PA0HRK, version 2.1)")
root.geometry("500x300")
root.config(cursor="none")

# Variable Label indicators
# Target selection: default = Moon 
v = IntVar()
v.set(1)
var=StringVar()
var.set("Moon")
# Auto Manual selection: default = Manual
vam = IntVar()
vam.set(2)
am_mode = StringVar()
am_mode.set("Manual")

space = [("Moon",1),("Sun",2),("Park",3),("Cold",4),("ISS",5)]
AM = [("Auto",1),("Manual",2)]

# apply target selected for display and related messages
def ShowChoice():			# 
    tgtsel = v.get()
    if tgtsel==1:
        var.set("Moon")
    elif tgtsel==2:
        var.set("Sun")
    elif tgtsel==3:
        var.set("Park")
    elif tgtsel==4:
        var.set("Cold")
    elif tgtsel==5:
        var.set("ISS")
        message.configure(text="Check TLE status")
        clear_message.configure(state = 'normal')
    else:
        print "fout1"     # abnormal situation
    return;
    
#  define Auto Manual Mode: 
def AM_procedure():
    global manual
    if vam.get()==1:
        am_mode.set("Auto")
        manual = False 
        for button in space_buttons:
            button.configure(state = 'disabled')
    else:
        am_mode.set("Manual")
        manual = True
        DMCC.setMotor(0,1,0)               # stop motors immediately
        DMCC.setMotor(0,2,0)
        elmov.configure(text="")           # and clear direction messages 
        azmov.configure(text="")
        for button in space_buttons:
            button.configure(state='normal')
    return;  

def clrmsg():                               # only purpose to clear message and disable clear button again
    message.configure(text="")              # if the messsage cause is removed it will stay away
    clear_message.configure(state = 'disabled')
    return;
 
# elevation motor controller: board 0; motor = 1; speed: 0-10000; direction: +/-
def el_motor_control(actual,setpoint):
    traject = abs(actual-setpoint)          # distance to travel
    sign = actual<setpoint                  # direction: left/right (boolean)
    if traject > 5:                         # speed 90%
        if sign:
            DMCC.setMotor(0,1,9000)
            elmov.configure(text="uuu")
        else:
            DMCC.setMotor(0,1,-9000)
            elmov.configure(text="ddd")
    elif traject >2:                        # speed 70%
        if sign:
            DMCC.setMotor(0,1,7000)
            elmov.configure(text="uu")
        else:
            DMCC.setMotor(0,1,-7000)
            elmov.configure(text="dd")
    elif traject > 0.11:                     # speed 50%
        if sign:
            DMCC.setMotor(0,1,5000)
            elmov.configure(text="u")
        else:
            DMCC.setMotor(0,1,-5000)
            elmov.configure(text="d")
    else:                                   # stop
        DMCC.setMotor(0,1,0)
        elmov.configure(text="-")
    return;
    
# azimuth motor controller: board 0; motor = 2; speed: 0-10000; direction: +/-
def az_motor_control(actual,setpoint):
    traject = abs(actual-setpoint)          # distance to travel
    sign = actual<setpoint                  # direction
    if traject > 5:                         # speed 60%
        if sign:
            DMCC.setMotor(0,2,6000)
            azmov.configure(text=">>>")
        else:
            DMCC.setMotor(0,2,-6000)
            azmov.configure(text="<<<")
    elif traject >2:                        # speed 40%
        if sign:
            DMCC.setMotor(0,2,4000)
            azmov.configure(text=">>")
        else:
            DMCC.setMotor(0,2,-4000)
            azmov.configure(text="<<")
    elif traject > 0.11:                     # speed 16%
        if sign:
            DMCC.setMotor(0,2,1600)
            azmov.configure(text=">")
        else:
            DMCC.setMotor(0,2,-1600)
            azmov.configure(text="<")
    else:                                   # stop
        DMCC.setMotor(0,2,0)
        azmov.configure(text="-")
    return;
    
# control antenna movement:  
def contant(az_sp,el_sp):
    ser4.open()
    read_byte = ser4.read(1)                # 6 byte datastring: delim\4 bytes data\termin (discard)
    if read_byte =="":                      # if nothing received: time out!
        message.configure(text= "serial sensor data time-out")
        clear_message.configure(state='normal')
    else: 
        while list(read_byte) <> delim:     # wait for start byte: delim 
            read_byte = ser4.read(1)
        data = ser4.read(5)                 # 4 bytes of position data + termination  
        term = list(data[4])                # termination (\x13) 
        if term <> termin:
            message.configure(text="sensor data corrupt")
            clear_message.configure(state='normal')
        el = (struct.unpack(">H",data[0:2])[0])*0.0879 - 90 # normalize incoming el data: horizontal==0 degrees
        az = (struct.unpack(">H",data[2:4])[0])*0.0879      # normalize incoming az data 360/4096
        el_ant = str('%5.1f' % el)
        elant.configure(text=el_ant)
        az_ant = str('%5.1f' % az)
        azant.configure(text=az_ant)  
    ser4.close()
    if not manual:                                  # Auto mode
        el_motor_control(el,el_sp+acemedat.elcor)   # elevation motor control (1)
        az_motor_control(az,az_sp+acemedat.azcor)   # azimut motor (2)
    else:                                           # Manual mode 
        button_state = 8*GPIO.input(up)+4*GPIO.input(dwn)+2*GPIO.input(lft)+GPIO.input(rgt)
        if button_state == 8:                       # Manual up 
            DMCC.setMotor(0,1,7000)                 # 70% of max speed 
            elmov.configure(text="u")
            DMCC.setMotor(0,2,0)
        elif button_state == 4:                     # Manual down
            DMCC.setMotor(0,1,-7000)
            elmov.configure(text="d")
            DMCC.setMotor(0,2,0)
        elif button_state == 2:                     # Manual left 
            DMCC.setMotor(0,1,0)                    # 50% of max speed 
            DMCC.setMotor(0,2,-5000)
            azmov.configure(text="<")
        elif button_state == 1:                     # Manual right 
            DMCC.setMotor(0,1,0)
            DMCC.setMotor(0,2,5000)
            azmov.configure(text=">")
        else:                                       # stop motors 
            DMCC.setMotor(0,1,0)
            DMCC.setMotor(0,2,0) 
            elmov.configure(text="")
            azmov.configure(text="")
    return;    

def quit_aceme():                                   # stop motors and leave program
    DMCC.setMotor(0,1,0)
    DMCC.setMotor(0,2,0)
    sys.exit()
    
# Fixed labels:
Label(root, text="Antenna", font=fnt).grid(row=1, column=1)
Label(root, text="Corr.", font=fnt, width=4).grid(row=1, column=3)
Label(root, text="Move", font=fnt, width=4).grid(row=1, column=4)
Label(root, text="Azimuth:", justify=RIGHT, font=fnt).grid(row=2, sticky=W)
Label(root, text="Elevation:", justify=RIGHT, font=fnt).grid(row=3, sticky=W) 
Label(root, text="Location:",justify=RIGHT, font=fnt).grid(row=4, sticky=W)
Label(root, text=acemedat.myloc, font=fnt).grid(row=4, column=1)
Label(root, text="Dat/Tim:", justify=RIGHT, font=fnt).grid(row=4, column=2) 
Label(root, text="Moon distance (km):", font=fnt).grid(row=5, columnspan=2, sticky=W)

def update_ephem():                                  # core business
    ob.lat, ob.long, ob.date = acemedat.mylat, acemedat.mylon, datetime.datetime.utcnow()
    my_date=str(ephem.now())
    timeText.configure(text=my_date)                 # read date/time from ephem 
    tgtsel = v.get()
    if tgtsel==1:                                    # Moon 
        obj=ephem.Moon(ob)
        az_setp=obj.az*180/pi 
        el_setp=obj.alt*180/pi
        caztrg=str('%4.1f' %az_setp)                 #    
        aztrg.configure(text=caztrg)
        celtrg=str('%4.1f' %el_setp)                 # to transfer data to labels
        eltrg.configure(text=celtrg)
    elif tgtsel==2:                                  # Sun 
        obj=ephem.Sun(ob)
        az_setp=obj.az*180/pi 
        el_setp=obj.alt*180/pi
        caztrg=str('%4.1f' %az_setp)                 #    
        aztrg.configure(text=caztrg)
        celtrg=str('%4.1f' %el_setp)                 # to transfer data to labels
        eltrg.configure(text=celtrg)
    elif tgtsel==3:                                  # Park
        az_setp=acemedat.azpark
        el_setp=acemedat.elpark
        caztrg=str(acemedat.azpark)
        aztrg.configure(text=caztrg)
        celtrg=str(acemedat.elpark)
        eltrg.configure(text=celtrg)        
    elif tgtsel==4:                                   # Cold (fixed)
        az_setp=acemedat.azcold
        el_setp=acemedat.elcold
        caztrg=str(acemedat.azcold)
        aztrg.configure(text=caztrg)
        celtrg=str(acemedat.elcold)
        eltrg.configure(text=celtrg) 
    elif tgtsel==5:                                   # ISS 
        iss.compute(ob)  
        az_setp=iss.az*180/pi 
        el_setp=iss.alt*180/pi
        caztrg=str('%4.1f' %az_setp)
        aztrg.configure(text=caztrg)
        celtrg=str('%4.1f' %el_setp)
        eltrg.configure(text=celtrg)         
    else:
        print "fout2: other"
    
    obj=ephem.Moon(ob)                                       # moondistance always on screen
    obj.compute(ob)
    moon_distance=str(int((obj.earth_distance*ephem.meters_per_au)/1000))   # distance in km
    Moondst.configure(text=moon_distance)
    contant(az_setp,el_setp)
    if GPIO.event_detected(stp):                            # Stop immediately and go in Manual mode
        vam.set(2)
        AM_procedure()
    root.after(200, update_ephem)                           # repeat cycle about 5 times per second
    
# Select Target
space_button_frame = LabelFrame(root, text= "Select Target:", bg="Yellow")
space_button_frame.grid(row =7, column=0, columnspan=5)
 
for txt,val in space:
    space_button=Radiobutton(space_button_frame, text = txt, font=fnt, indicatoron=0, width=8, variable=v, command=ShowChoice,value=val)
    space_button.grid(row=7, column=val-1)
    space_buttons.append(space_button)                       # fill space button list for later reference

am_button_frame=LabelFrame(root, text="Control Mode:", bg="Yellow")
am_button_frame.grid(row=8, column=0, columnspan = 2, sticky = W)
    
for txt,val in AM:
    am_button=Radiobutton(am_button_frame, text = txt, font=fnt, indicatoron=0, width=8, variable=vam, command=AM_procedure, value=val)
    am_button.grid(row=8, column=val-1)
 
# Variable labels containing data to be displayed:
target_label = Label(root, textvariable=var, font=fnt)
target_label.grid(row=1, column=2) 

aztrg = Label(root, text="", font=fnt)
aztrg.grid(row=2, column=2)
eltrg = Label(root, text="", font=fnt)
eltrg.grid(row=3, column=2)

azant = Label(root, text="", font=fnt)
azant.grid(row=2, column=1)
elant = Label(root, text="", font=fnt)
elant.grid(row=3, column=1)

az_cor = Label(root, text=str(acemedat.azcor), font=fnt)
az_cor.grid(row=2, column=3)
el_cor = Label(root, text=str(acemedat.elcor), font=fnt)
el_cor.grid(row=3, column =3)

azmov = Label(root, text="", font=fnt)
azmov.grid(row=2, column=4)
elmov = Label(root, text="", font=fnt)
elmov.grid(row=3, column=4)

timeText = Label(root, text=" ",font=fnt)
timeText.grid(row=4, column=3, sticky=W, columnspan=3)

Moondst = Label(root, text="", font=fnt)
Moondst.grid(row=5, column=2, sticky=W)

clear_message = Checkbutton(root, text="Clear:", font=fnt, indicatoron=0, width=8, command=clrmsg, state='disabled')
clear_message.grid(row=9, column=0, sticky=W)

message = Label(root, text= "", font=fnt, fg = "red")
message.grid(row=9, column=1, columnspan=4, sticky=W)

menu_frame=LabelFrame(root, text= "Exit Program", bg = "Red")
menu_frame.grid(row=8, column=4, sticky=E)
menu = Checkbutton(menu_frame, text= "Quit", font=fnt, indicatoron=0, width=8, command=quit_aceme).grid(row=8, column=4, sticky=W+E+N+S)

update_ephem()
root.update()
root.mainloop()

